from abc import ABC

from gql.gql import gql
from gql.transport.requests import RequestsHTTPTransport
from bergen.wards.base import BaseWard, WardException
from bergen.query import GQL, TypedGQL
from gql.transport.aiohttp import AIOHTTPTransport
import logging
from gql.transport.aiohttp import log as aiohttp_logger
from gql.transport.requests import log as requests_logger
aiohttp_logger.setLevel(logging.WARNING)
requests_logger.setLevel(logging.WARNING)
import asyncio
from gql import Client, gql


class GraphQLException(WardException):
    pass


class BaseGraphQLWard(BaseWard, ABC):
    can_subscribe = False
    

    def __init__(self, port=8000, host="localhost", protocol="http", token=None, loop = None) -> None:
        self._graphql_endpoint = f"{protocol}://{host}:{port}/graphql"
        self._token = token
        self._headers = {"Authorization": f"Bearer {self._token}"}
        
        self.sync_transport = RequestsHTTPTransport(self._graphql_endpoint, headers=self._headers, verify=True, retries=3)
        self.sync_transport.connect()
        
        super().__init__(loop=loop)

    
    def run(self, the_query: TypedGQL, variables: dict = {}, **kwargs):
        query_node = gql(the_query.query)
        response = self.sync_transport.execute(query_node, variable_values=variables)
        if response.errors:
            raise GraphQLException(f"Error: {self._graphql_endpoint} {str(response.errors)}")
        return the_query.extract(response.data)




        



from bergen.query import ListQuery, QueryList, TypedGQL
from typing import Any, Optional
from bergen.types.object import ArnheimObject
from bergen.schema import Port
from bergen.ui.widgets.base import BaseWidgetMixin
from PyQt5.QtWidgets import QComboBox, QHBoxLayout, QWidget
from bergen.registries.arnheim import get_current_arnheim

class SelectorItem(ArnheimObject):
    value: Optional[Any]
    label: Optional[str]




class QueryWidget(BaseWidgetMixin, QWidget):
    query = None


    def __init__(self, *args, query=None, identifier=None, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.query = self.query or query
        assert self.query is not None

        self.ward = get_current_arnheim().getWardForIdentifier(identifier)
        self.model_selector = QComboBox()
        self.model_selector.currentIndexChanged.connect(self.indexChanged)
        self.layout = QHBoxLayout()
        self.layout.addWidget(self.model_selector)
        self.current_value = None
        self.setLayout(self.layout)


    def indexChanged(self, index):
        value = self.items[index].value
        if value == self.current_value:
            return

        self.current_value = value
        print(self.current_value)
        if self.on_changed: self.on_changed(value)


    def render(self, keyValueMap):
        self.model_selector.clear()
        self.items = QueryList(self.query, SelectorItem).run(ward=self.ward, variables=keyValueMap)
        for item in self.items:
            self.model_selector.addItem(item.label)
        
            
    @classmethod
    def fromPort(cls, port: Port, **kwargs):
        assert port.widget.query is not None, "Please request query on QueryWidgetType to build a QueryWidget"
        return cls(port=port, query=port.widget.query, identifier = port.identifier, **kwargs)





from bergen.enums import ClientType
from bergen.auths.types import GrantType
from bergen.clients.default import Bergen
import os
from bergen.auths.legacy.app import LegacyApplication
from bergen.auths.backend.app import ArnheimBackendOauth
from bergen.auths.implicit.app import ImplicitApplication


class HostBergen(Bergen):
    pass
    
    def __init__(self, *args, scopes= [],**kwargs) -> None:
        super().__init__(*args, client_type = ClientType.HOST, scopes=scopes + ["host"], **kwargs)


from bergen.debugging import DebugLevel
from bergen.handlers.base import ContractHandler
from bergen.messages import BouncedForwardedAssignMessage, AssignYieldsMessage, AssignProgressMessage, AssignDoneMessage, AssignReturnMessage, AssignCriticalMessage
from bergen.console import console


class AssignHandler(ContractHandler[BouncedForwardedAssignMessage]):

    @property
    def meta(self):
        return {"extensions": self.message.meta.extensions, "reference": self.message.meta.reference}

    async def pass_progress(self, text: str = None, percentage: int = None):
        console.log(text + percentage)

    async def pass_yield(self, value):
        yield_message = AssignYieldsMessage(data={"returns": value}, meta=self.meta)
        await self.forward(yield_message)

    async def pass_progress(self, value, percentage=None):
        value = f'{percentage if percentage else "--"} : {value}'
        progress_message = AssignProgressMessage(data={"message": value, "level": "INFO"}, meta=self.meta)
        await self.forward(progress_message)

    async def pass_done(self):
        return_message = AssignDoneMessage(data={"ok": True}, meta=self.meta)
        await self.forward(return_message)

    async def pass_result(self, value):
        return_message = AssignReturnMessage(data={"returns": value}, meta=self.meta)
        await self.forward(return_message)

    async def pass_exception(self, exception):
        error_message = AssignCriticalMessage(data={"message": str(exception), "type": str(exception.__class__.__name__)}, meta=self.meta)
        await self.forward(error_message)